@@ -7,8 +7,8 @@ import pyperf
 from pyperf import tests
 
 
-TELCO = os.path.join(os.path.dirname(__file__), 'telco.json')
-TRACK_MEMORY = os.path.join(os.path.dirname(__file__), 'track_memory.json')
+TESTDIR = os.path.dirname(__file__)
+TELCO = os.path.join(TESTDIR, 'telco.json')
 
 
 class BaseTestCase(object):
@@ -129,7 +129,7 @@ class TestPerfCLI(BaseTestCase, unittest.TestCase):
         stdout = self.compare('compare_to', ref_result, changed_result, '-v')
 
         expected = ('Mean +- std dev: [ref] 1.50 sec +- 0.50 sec '
-                    '-> [changed] 2.00 sec +- 0.50 sec: 1.33x slower (+33%)\n'
+                    '-> [changed] 2.00 sec +- 0.50 sec: 1.33x slower\n'
                     'Not significant!')
         self.assertEqual(stdout.rstrip(),
                          expected)
@@ -144,11 +144,11 @@ class TestPerfCLI(BaseTestCase, unittest.TestCase):
         stdout = self.compare('compare_to', ref_result, changed_result, '--table')
 
         expected = textwrap.dedent('''
-            +-----------+----------+--------------------------------+
-            | Benchmark | ref      | changed                        |
-            +===========+==========+================================+
-            | telco     | 1.00 sec | 2.00 sec: 2.00x slower (+100%) |
-            +-----------+----------+--------------------------------+
+            +-----------+----------+------------------------+
+            | Benchmark | ref      | changed                |
+            +===========+==========+========================+
+            | telco     | 1.00 sec | 2.00 sec: 2.00x slower |
+            +-----------+----------+------------------------+
         ''').strip()
 
         self.assertEqual(stdout.rstrip(),
@@ -162,7 +162,7 @@ class TestPerfCLI(BaseTestCase, unittest.TestCase):
                                            metadata={'name': 'telco'})
 
         stdout = self.compare('compare_to', ref_result, changed_result, '--table')
-        expected = "Not significant (1): telco"
+        expected = "Benchmark hidden because not significant (1): telco"
         self.assertEqual(stdout.rstrip(),
                          expected)
 
@@ -187,7 +187,7 @@ class TestPerfCLI(BaseTestCase, unittest.TestCase):
         stdout = self.compare('compare_to', ref_result, changed_result, '-v')
 
         expected = ('Mean +- std dev: [ref] 1.50 sec +- 0.50 sec '
-                    '-> [changed] 2.00 sec +- 0.50 sec: 1.33x slower (+33%)\n'
+                    '-> [changed] 2.00 sec +- 0.50 sec: 1.33x slower\n'
                     'Not significant!')
         self.assertEqual(stdout.rstrip(),
                          expected)
@@ -207,7 +207,150 @@ class TestPerfCLI(BaseTestCase, unittest.TestCase):
 
     def check_command(self, expected, *args, **kwargs):
         stdout = self.run_command(*args, **kwargs)
-        self.assertEqual(stdout.rstrip(), textwrap.dedent(expected).strip())
+        self.assertEqual(stdout, textwrap.dedent(expected).lstrip())
+
+    def test_compare_to_cli(self):
+        py36 = os.path.join(TESTDIR, 'mult_list_py36.json')
+        py37 = os.path.join(TESTDIR, 'mult_list_py37.json')
+        py38 = os.path.join(TESTDIR, 'mult_list_py38.json')
+
+        # 2 files
+        expected = """
+            [1]*1000: Mean +- std dev: [mult_list_py36] 2.13 us +- 0.06 us -> [mult_list_py37] 2.09 us +- 0.04 us: 1.02x faster
+            [1,2]*1000: Mean +- std dev: [mult_list_py36] 3.70 us +- 0.05 us -> [mult_list_py37] 5.28 us +- 0.09 us: 1.42x slower
+            [1,2,3]*1000: Mean +- std dev: [mult_list_py36] 4.61 us +- 0.13 us -> [mult_list_py37] 6.05 us +- 0.11 us: 1.31x slower
+
+            Geometric mean: 1.22x slower
+        """
+        self.check_command(expected, 'compare_to', py36, py37)
+
+        # 2 files grouped by speed
+        expected = """
+            Slower (2):
+            - [1,2]*1000: 3.70 us +- 0.05 us -> 5.28 us +- 0.09 us: 1.42x slower
+            - [1,2,3]*1000: 4.61 us +- 0.13 us -> 6.05 us +- 0.11 us: 1.31x slower
+
+            Faster (1):
+            - [1]*1000: 2.13 us +- 0.06 us -> 2.09 us +- 0.04 us: 1.02x faster
+
+            Geometric mean: 1.22x slower
+        """
+        self.check_command(expected, 'compare_to', "--group-by-speed", py36, py37)
+
+        # 2 files grouped by speed (with not significant)
+        expected = """
+            Faster (2):
+            - [1,2]*1000: 3.70 us +- 0.05 us -> 3.18 us +- 0.08 us: 1.16x faster
+            - [1,2,3]*1000: 4.61 us +- 0.13 us -> 4.17 us +- 0.11 us: 1.11x faster
+
+            Benchmark hidden because not significant (1): [1]*1000
+
+            Geometric mean: 1.09x faster
+        """
+        self.check_command(expected, 'compare_to', "--group-by-speed", py36, py38)
+
+        # 3 files
+        expected = """
+            [1]*1000
+            ========
+
+            Mean +- std dev: [mult_list_py36] 2.13 us +- 0.06 us -> [mult_list_py37] 2.09 us +- 0.04 us: 1.02x faster
+            Mean +- std dev: [mult_list_py36] 2.13 us +- 0.06 us -> [mult_list_py38] 2.13 us +- 0.03 us: 1.00x slower
+            Not significant!
+
+            [1,2]*1000
+            ==========
+
+            Mean +- std dev: [mult_list_py36] 3.70 us +- 0.05 us -> [mult_list_py37] 5.28 us +- 0.09 us: 1.42x slower
+            Mean +- std dev: [mult_list_py36] 3.70 us +- 0.05 us -> [mult_list_py38] 3.18 us +- 0.08 us: 1.16x faster
+
+            [1,2,3]*1000
+            ============
+
+            Mean +- std dev: [mult_list_py36] 4.61 us +- 0.13 us -> [mult_list_py37] 6.05 us +- 0.11 us: 1.31x slower
+            Mean +- std dev: [mult_list_py36] 4.61 us +- 0.13 us -> [mult_list_py38] 4.17 us +- 0.11 us: 1.11x faster
+
+            Geometric mean
+            ==============
+
+            mult_list_py37: 1.22x slower
+            mult_list_py38: 1.09x faster
+        """
+        self.check_command(expected, 'compare_to', py36, py37, py38)
+
+        # 3 files as table
+        expected = """
+            +----------------+----------------+-----------------------+-----------------------+
+            | Benchmark      | mult_list_py36 | mult_list_py37        | mult_list_py38        |
+            +================+================+=======================+=======================+
+            | [1]*1000       | 2.13 us        | 2.09 us: 1.02x faster | not significant       |
+            +----------------+----------------+-----------------------+-----------------------+
+            | [1,2]*1000     | 3.70 us        | 5.28 us: 1.42x slower | 3.18 us: 1.16x faster |
+            +----------------+----------------+-----------------------+-----------------------+
+            | [1,2,3]*1000   | 4.61 us        | 6.05 us: 1.31x slower | 4.17 us: 1.11x faster |
+            +----------------+----------------+-----------------------+-----------------------+
+            | Geometric mean | (ref)          | 1.22x slower          | 1.09x faster          |
+            +----------------+----------------+-----------------------+-----------------------+
+        """
+        self.check_command(expected, 'compare_to', '--table', py36, py37, py38)
+
+        # 3 files as table grouped by speed
+        expected = """
+            +----------------+----------------+-----------------------+
+            | Benchmark      | mult_list_py36 | mult_list_py37        |
+            +================+================+=======================+
+            | [1]*1000       | 2.13 us        | 2.09 us: 1.02x faster |
+            +----------------+----------------+-----------------------+
+            | [1,2,3]*1000   | 4.61 us        | 6.05 us: 1.31x slower |
+            +----------------+----------------+-----------------------+
+            | [1,2]*1000     | 3.70 us        | 5.28 us: 1.42x slower |
+            +----------------+----------------+-----------------------+
+            | Geometric mean | (ref)          | 1.22x slower          |
+            +----------------+----------------+-----------------------+
+        """
+        self.check_command(expected, 'compare_to', '--table', "--group-by-speed", py36, py37)
+
+    def test_compare_to_cli_min_speed(self):
+        py36 = os.path.join(TESTDIR, 'mult_list_py36.json')
+        py37 = os.path.join(TESTDIR, 'mult_list_py37.json')
+        py38 = os.path.join(TESTDIR, 'mult_list_py38.json')
+
+        # 2 files, min-speed=10
+        expected = """
+            [1,2]*1000: Mean +- std dev: [mult_list_py36] 3.70 us +- 0.05 us -> [mult_list_py37] 5.28 us +- 0.09 us: 1.42x slower
+            [1,2,3]*1000: Mean +- std dev: [mult_list_py36] 4.61 us +- 0.13 us -> [mult_list_py37] 6.05 us +- 0.11 us: 1.31x slower
+
+            Benchmark hidden because not significant (1): [1]*1000
+
+            Geometric mean: 1.22x slower
+        """
+        self.check_command(expected, 'compare_to', "--min-speed=10", py36, py37)
+
+        # 2 files, min-speed=40
+        expected = """
+            [1,2]*1000: Mean +- std dev: [mult_list_py36] 3.70 us +- 0.05 us -> [mult_list_py37] 5.28 us +- 0.09 us: 1.42x slower
+
+            Benchmark hidden because not significant (2): [1]*1000, [1,2,3]*1000
+
+            Geometric mean: 1.22x slower
+        """
+        self.check_command(expected, 'compare_to', "--min-speed=40", py36, py37)
+
+        # 3 files as table, min-speed=10
+        expected = """
+            +----------------+----------------+-----------------------+-----------------------+
+            | Benchmark      | mult_list_py36 | mult_list_py37        | mult_list_py38        |
+            +================+================+=======================+=======================+
+            | [1,2]*1000     | 3.70 us        | 5.28 us: 1.42x slower | 3.18 us: 1.16x faster |
+            +----------------+----------------+-----------------------+-----------------------+
+            | [1,2,3]*1000   | 4.61 us        | 6.05 us: 1.31x slower | 4.17 us: 1.11x faster |
+            +----------------+----------------+-----------------------+-----------------------+
+            | Geometric mean | (ref)          | 1.22x slower          | 1.09x faster          |
+            +----------------+----------------+-----------------------+-----------------------+
+
+            Benchmark hidden because not significant (1): [1]*1000
+        """
+        self.check_command(expected, 'compare_to', '--table', "--min-speed=10", py36, py37, py38)
 
     def test_hist(self):
         # Force terminal size on Python 3 for shutil.get_terminal_size()
@@ -325,7 +468,8 @@ class TestPerfCLI(BaseTestCase, unittest.TestCase):
             Run 4: 0 warmups, 1 value, 2^15 loops
             - value 1: 7208.0 kB
         """
-        stdout = self.run_command('dump', TRACK_MEMORY)
+        filename = os.path.join(TESTDIR, 'track_memory.json')
+        stdout = self.run_command('dump', filename)
         self.assertIn(textwrap.dedent(expected).strip(), stdout)
 
     def test_dump_quiet(self):
